open Pendulum_data
open Misc

let t = 0.05  (* s     *)
let l = 10.0 (* m     *)
let g = 10.0 (* m/s^2 *)
let b = 0.1  (* 1/s *)

let integr dx = Misc.integr t dx
let deriv x = Misc.deriv t x

(*
  equation of the pendulum	:
  - theta angle of the pendulum with Oy
  - d2x0,d2y0 second degree derivatives of the botom coordinates
  - l length of the pendulum
  - g acceleration  *)    
let pendulum d2x0 d2y0 = theta where 
    rec theta = integr (integr ((sin thetap)*.(d2y0 +. g) 
				  -.(cos thetap)*.d2x0)/.l)
    and thetap = 0.0 fby theta
        
    
(* second version with loss in energy *)
let pendulum d2x0 d2y0 = theta where 
    rec theta = integr (integr ((sin thetap)*.(d2y0 +. g) 
				  -.(cos thetap)*.d2x0)/.l
			  -. b *. thetap)
    and thetap = 0.0 fby theta
   

let game x0 y0 = 


(* pendulum controled by hand:        *)
(* x0, y0 are the botom coordinates   *)
    
(* derivation *)

  let d2x0 = deriv (deriv x0)  in
  let d2y0 = deriv (deriv y0)  in

(* pendulum *)

  let theta = pendulum d2x0 d2y0 in

(* geometry *)

  let x = x0 +. l*.(sin theta)  in
  let y = y0 +. l*.(cos theta)  in
  make_pend x0 y0 x y


(* controled pendulum *)
(* delta is the initial acceleration given to the pendulum *)
let controled delta = 
  let rec d2y0 = 0.0
  and (* regulation law *)
      d2x0 = delta ->( 8.0 *. g *. sin(teta) /. cos(teta)
                      +. sqrt(1.0 *. g *. l) *. deriv(teta) 
                      +. 0.5 *. x0 /. l)
  and x0 = integr(integr(delta fby d2x0))
  and y0 = integr(integr(d2y0))
      (* pendulum *)
  and teta = pendulum (delta fby d2x0) d2y0 in
      (* geometry *)
  let x  = x0 +. l *. sin(teta) in
  let y  = y0 +. l *. cos(teta) in
  let p  = make_pend (x0 +. 20.0) (y0 +. 20.0) (x +. 20.0) (y +. 20.0) in
  p;;


let manual() =
  let x0,y0 = get_cursor() in
  let p = game x0 y0 in
  draw_pendulum p (p -> pre p );;


let auto() =
  let p = controled 10.0 in
  draw_pendulum p (p fby p);;

